@@ -1,44 +1,50 @@
 import base64
-
 import collections.abc
+import os
 import sys
 import time
+from datetime import date
 
 import gitlab
-import os
-from datetime import date
-from minio import Minio
 from kubernetes import client, config
+from minio import Minio
+
 ######################################
 # Python wrapping nestor
 ######################################
-from .pynestor import NestorDescSet, ScriptNestorInstance, NestorGitDesc, Utils, NestorOpt
+from .pynestor import (
+    NestorDescSet,
+    NestorGitDesc,
+    NestorOpt,
+    ScriptNestorInstance,
+    Utils,
+)
 
 CURRENT_PATH = os.path.expanduser(os.path.abspath(os.path.curdir))
 
-class EnvironementConfig():
 
+class EnvironementConfig:
     def __init__(self, base_env_vars=None):
         base_env_vars = dict(base_env_vars or {}, **os.environ)
-        self.NESTOR_NAME = base_env_vars.get('NESTOR_NAME') or base_env_vars.get('CI_COMMIT_REF_SLUG')
+        self.NESTOR_NAME = base_env_vars.get("NESTOR_NAME") or base_env_vars.get("CI_COMMIT_REF_SLUG")
         self.ALWAYS_DELETE = base_env_vars.get("ALWAYS_DELETE")
-        self.CI_COMMIT_REF_NAME = base_env_vars.get('CI_COMMIT_REF_NAME')
-        self.CI_PROJECT_DIR = base_env_vars.get('CI_PROJECT_DIR')
-        self.CI_PROJECT_PATH = base_env_vars.get('CI_PROJECT_PATH')
-        self.GITLAB_TOKEN = base_env_vars.get('GITLAB_TOKEN')
-        self.ODOO_VERSION = base_env_vars.get('ODOO_VERSION')
-        self.S3_SECRET_PREVIEW = base_env_vars.get('S3_SECRET_PREVIEW')
-        self.S3_DUMP_SECRET = base_env_vars.get('S3_DUMP_SECRET')
-        self.S3_DUMP_BUCKET = base_env_vars.get('S3_DUMP_BUCKET')
+        self.CI_COMMIT_REF_NAME = base_env_vars.get("CI_COMMIT_REF_NAME")
+        self.CI_PROJECT_DIR = base_env_vars.get("CI_PROJECT_DIR")
+        self.CI_PROJECT_PATH = base_env_vars.get("CI_PROJECT_PATH")
+        self.GITLAB_TOKEN = base_env_vars.get("GITLAB_TOKEN")
+        self.ODOO_VERSION = base_env_vars.get("ODOO_VERSION")
+        self.S3_SECRET_PREVIEW = base_env_vars.get("S3_SECRET_PREVIEW")
+        self.S3_DUMP_SECRET = base_env_vars.get("S3_DUMP_SECRET")
+        self.S3_DUMP_BUCKET = base_env_vars.get("S3_DUMP_BUCKET")
         self.VERBOSE = bool(base_env_vars.get("VERBOSE"))
-        self.DATE = base_env_vars.get('DATE')
-        self.S3_BUCKET = base_env_vars.get('S3_BUCKET')
-        self.PROD_INSTANCE_NAME = base_env_vars.get('PROD_INSTANCE_NAME') or base_env_vars.get('CI_PROJECT_NAME')
-        self.DB_NAME = base_env_vars.get('DB_NAME')
-        self.ODOO_DEPENDS = base_env_vars.get('ODOO_DEPENDS')
+        self.DATE = base_env_vars.get("DATE")
+        self.S3_BUCKET = base_env_vars.get("S3_BUCKET")
+        self.PROD_INSTANCE_NAME = base_env_vars.get("PROD_INSTANCE_NAME") or base_env_vars.get("CI_PROJECT_NAME")
+        self.DB_NAME = base_env_vars.get("DB_NAME")
+        self.ODOO_DEPENDS = base_env_vars.get("ODOO_DEPENDS")
         self.DUMP_PATH = base_env_vars.get("PATH_DUMP")
-        self.NESTOR_CFG_FILENAME = os.environ.get('NESTOR_CFG_FILENAME', '.nestor-cfg.yml')
-        self.BASE_MODULE_UPDATE = os.environ.get('BASE_MODULE_UPDATE', 'all')
+        self.NESTOR_CFG_FILENAME = os.environ.get("NESTOR_CFG_FILENAME", ".nestor-cfg.yml")
+        self.BASE_MODULE_UPDATE = os.environ.get("BASE_MODULE_UPDATE", "all")
 
     def apply_default(self):
 
@@ -49,16 +55,16 @@ class EnvironementConfig():
         self.DATE = self.DATE or date.today().isoformat()
         self.S3_BUCKET = self.S3_BUCKET or self.NESTOR_NAME
         self.DB_NAME = self.DB_NAME or self.PROD_INSTANCE_NAME
-        self.ODOO_DEPENDS = (self.ODOO_DEPENDS or ('odoo-addons/common-modules,odoo-addons/community-addons'))
+        self.ODOO_DEPENDS = self.ODOO_DEPENDS or ("odoo-addons/common-modules,odoo-addons/community-addons")
         dump_path_fmt = "%(PROD_INSTANCE_NAME)s/journaliere/%(DB_NAME)s_%(DATE)s.dump"
         self.DUMP_PATH = (self.DUMP_PATH or dump_path_fmt) % {
             "PROD_INSTANCE_NAME": self.PROD_INSTANCE_NAME,
             "DB_NAME": self.DB_NAME,
             "DATE": self.DATE,
         }
-        self.NESTOR_CFG_FILENAME = self.NESTOR_CFG_FILENAME or '.nestor-cfg.yml'
-        self.BASE_MODULE_UPDATE = self.BASE_MODULE_UPDATE or 'all'
-        self.ODOO_DEPENDS = self.ODOO_DEPENDS or 'odoo-addons/common-modules,odoo-addons/community-addons'
+        self.NESTOR_CFG_FILENAME = self.NESTOR_CFG_FILENAME or ".nestor-cfg.yml"
+        self.BASE_MODULE_UPDATE = self.BASE_MODULE_UPDATE or "all"
+        self.ODOO_DEPENDS = self.ODOO_DEPENDS or "odoo-addons/common-modules,odoo-addons/community-addons"
         self.S3_SECRET_PREVIEW = self.S3_SECRET_PREVIEW or "s3-ndp-preview"
         self.S3_DUMP_SECRET = self.S3_DUMP_SECRET or "s3-ndp-scaleway"
         self.S3_DUMP_BUCKET = self.S3_DUMP_BUCKET or "ndp-production-backups"
@@ -89,7 +95,7 @@ class EnvironementConfig():
 class InteractiveConfig(EnvironementConfig):
     def __init__(self):
         super(InteractiveConfig, self).__init__()
-        log(f"Nom du projet git (Sans le 'odoo-addons/' ex: mettre `aef` pour `odoo-addons/aef`) ?")
+        log("Nom du projet git (Sans le 'odoo-addons/' ex: mettre `aef` pour `odoo-addons/aef`) ?")
         log("Mettre un '/' si c'est un path absolut (ex /odoo/v15/tms/core')")
         self.CI_PROJECT_PATH = input("=> ")
         DEFAULT_PROD_INSTANCE_NAME = self.CI_PROJECT_PATH
@@ -100,7 +106,7 @@ class InteractiveConfig(EnvironementConfig):
             log("Un projet est obligatoire")
             sys.exit(1)
 
-        log(f"Version d'Odoo ?")
+        log("Version d'Odoo ?")
         self.ODOO_VERSION = input("=> ")
 
         log(f"Nom de la branche ou du tag || default = {self.ODOO_VERSION}")
@@ -109,11 +115,9 @@ class InteractiveConfig(EnvironementConfig):
         log(f"Nom du nestor (Vide pour utliser '{self.CI_COMMIT_REF_NAME}')")
         self.NESTOR_NAME = input("=> ") or self.CI_COMMIT_REF_NAME
 
-
         log(f"Nom de l'instance nestor dans le kube de prod || default '{DEFAULT_PROD_INSTANCE_NAME}'")
         self.PROD_INSTANCE_NAME = input("=> ") or DEFAULT_PROD_INSTANCE_NAME
 
-
         log(f"Repertoire du projet ou laisser vide pour utiliser '{CURRENT_PATH}'")
         self.CI_PROJECT_DIR = input("=> ") or CURRENT_PATH
 
@@ -132,7 +136,6 @@ def log(*args):
 
 
 class PreviewUtils:
-
     def __init__(self, gitlab_api, env_config):
         self.gitlab_api = gitlab_api
         self.env_config = env_config
@@ -158,21 +161,30 @@ class PreviewUtils:
         current_gl_project = self.gitlab_api.projects.get(main_project_path)
         default_branch = Utils.quote_if_needed(str(current_gl_project.default_branch))
         nestor_values = NestorDescSet([NestorOpt("sources.branch", default_branch)])
-        spec_src = self.create_spec_sources(idx=0, project_path=main_project_path, branch_to_test=[
-            self.env_config.CI_COMMIT_REF_NAME,
-            current_gl_project.default_branch,
-            self.env_config.ODOO_VERSION
-        ])
+        spec_src = self.create_spec_sources(
+            idx=0,
+            project_path=main_project_path,
+            branch_to_test=[
+                self.env_config.CI_COMMIT_REF_NAME,
+                current_gl_project.default_branch,
+                self.env_config.ODOO_VERSION,
+            ],
+        )
         nestor_values.add(spec_src)
         idx = 2
         for depend in self.env_config.ODOO_DEPENDS.split(","):
             if depend:
                 nestor_values.add(
-                    self.create_spec_sources(idx=idx, project_path=depend, branch_to_test=[
-                        self.env_config.CI_COMMIT_REF_NAME,
-                        current_gl_project.default_branch,
-                        self.env_config.ODOO_VERSION
-                    ]))
+                    self.create_spec_sources(
+                        idx=idx,
+                        project_path=depend,
+                        branch_to_test=[
+                            self.env_config.CI_COMMIT_REF_NAME,
+                            current_gl_project.default_branch,
+                            self.env_config.ODOO_VERSION,
+                        ],
+                    )
+                )
                 idx += 1
         return nestor_values
 
@@ -189,7 +201,7 @@ class PreviewUtils:
 
     def _get_spec_from_path(self, nestor_cfg_path, sections, stage):
         values = NestorDescSet()
-        log('')
+        log("")
         log("Use cfg file %s" % nestor_cfg_path)
         if os.path.exists(nestor_cfg_path):
             for section in sections:
@@ -199,12 +211,12 @@ class PreviewUtils:
             log("Cfg file %s not exist" % nestor_cfg_path)
         return values
 
-class ScriptNestor():
 
+class ScriptNestor:
     def __init__(self, config: EnvironementConfig):
         self.config = config
         self.inst = ScriptNestorInstance(config.NESTOR_NAME)
-        self.gitlab_api = gitlab.Gitlab("https://gitlab.ndp-systemes.fr", config.GITLAB_TOKEN, api_version='4')
+        self.gitlab_api = gitlab.Gitlab("https://gitlab.ndp-systemes.fr", config.GITLAB_TOKEN, api_version="4")
         self.preview_utils = PreviewUtils(self.gitlab_api, self.config)
 
     def delete(self):
@@ -215,16 +227,16 @@ class ScriptNestor():
     def edit_with_values(self, values):
         log("MAJ de l'instance", self.inst.name)
         log(values.to_str(pretty=True))
-        log('')
+        log("")
         self.stop()
         self.inst.delete_and_exit_if_failed(self.inst.edit(values))
         time.sleep(10)
 
     def create_with_values(self, values):
-        log('')
+        log("")
         log("Nouvelle instance", self.config.NESTOR_NAME)
         log(values.to_str(pretty=True))
-        log('')
+        log("")
         self.inst.create(self.config.ODOO_VERSION, values_set=values)
         self.wait_all_up()
 
@@ -241,24 +253,46 @@ class ScriptNestor():
         self.inst.delete_and_exit_if_failed(self.inst.wait(up=False, postgres=False, timeout=5 * 60))  # Timeout de 5min
 
     def enable_s3_spec(self, values):
-        if values['persistence.s3.secret']:
-            values.add(self.inst.filestore.enable_on_s3_spec(s3_secret="s3-ndp-preview", s3_bucket=self.config.S3_BUCKET, values=values))
+        if values["persistence.s3.secret"]:
+            values.add(
+                self.inst.filestore.enable_on_s3_spec(
+                    s3_secret="s3-ndp-preview", s3_bucket=self.config.S3_BUCKET, values=values
+                )
+            )
+        client = self._get_s3_client()
+        if not client.bucket_exists(self.config.S3_BUCKET):
+            client.make_bucket(self.config.S3_BUCKET)
         return values
 
-
     def log_spec(self, spec_values: NestorDescSet):
-        log('')
+        log("")
         log("Spec Nestor", self.inst.name)
         log(spec_values.to_str(pretty=True))
-        log('')
+        log("")
 
     def run_script(self) -> int:
         log("Nothing to do !!!!!!!!!!")
         return 1
 
+    def _get_s3_client(self):
+        try:
+            config.load_incluster_config()
+        except config.config_exception.ConfigException:
+            config.load_config()
+        v1 = client.CoreV1Api()
+        ret = v1.list_namespaced_secret("ndp-test")
+        for i in ret.items:
+            if (i.metadata.labels or {}).get("type") == "s3":
+                if self.config.S3_SECRET_PREVIEW == i.metadata.name:
+                    return Minio(
+                        endpoint=base64.b64decode(i.data["S3_HOST"]).decode("utf-8"),
+                        access_key=base64.b64decode(i.data["S3_ACCESS_KEY"]).decode("utf-8"),
+                        secret_key=base64.b64decode(i.data["S3_SECRET_KEY"]).decode("utf-8"),
+                        region=base64.b64decode(i.data["S3_REGION"]).decode("utf-8"),
+                    )
+
 
 class PreviewUpScript(ScriptNestor):
-
     def run_script(self):
         if not self.inst.name:
             log("Instance don't have a name")
@@ -270,11 +304,9 @@ class PreviewUpScript(ScriptNestor):
 
         if not self.inst.exist():
             log("Create instance")
-            values = self.preview_utils.get_spec_values(sections=[
-                "default",
-                "preview",
-                self.config.CI_COMMIT_REF_NAME
-            ], stage="restore")
+            values = self.preview_utils.get_spec_values(
+                sections=["default", "preview", self.config.CI_COMMIT_REF_NAME], stage="restore"
+            )
             values.add(self.preview_utils._create_all_spec_sources(self.config.CI_PROJECT_PATH))
             values.add(self.enable_s3_spec(values))
 
@@ -289,17 +321,15 @@ class PreviewUpScript(ScriptNestor):
                     self.config.DUMP_PATH,
                     s3_secret=self.config.S3_DUMP_SECRET,
                     bucket=self.config.S3_DUMP_BUCKET,
-                    set_password_to_all=True
+                    set_password_to_all=True,
                 )
             )
         else:
             log("MAJ de l'instance", self.inst.name)
             self.stop()
-        values = self.preview_utils.get_spec_values(sections=[
-            "default",
-            "preview",
-            self.config.CI_COMMIT_REF_NAME
-        ], stage="runtime")
+        values = self.preview_utils.get_spec_values(
+            sections=["default", "preview", self.config.CI_COMMIT_REF_NAME], stage="runtime"
+        )
         values = self.enable_s3_spec(values)
         self.log_spec(values)
         self.edit_with_values(values)
@@ -322,7 +352,6 @@ class PreviewUpScript(ScriptNestor):
 
 
 class PreviewDownScript(ScriptNestor):
-
     def run_script(self):
         s3_client = self._get_s3_client()
         self.inst.delete()
@@ -334,20 +363,3 @@ class PreviewDownScript(ScriptNestor):
             s3_client.remove_bucket(self.config.S3_BUCKET)
         log("OK")
         return 0
-
-    def _get_s3_client(self):
-        try:
-            config.load_incluster_config()
-        except config.config_exception.ConfigException:
-            config.load_config()
-        v1 = client.CoreV1Api()
-        ret = v1.list_namespaced_secret("ndp-test")
-        for i in ret.items:
-            if (i.metadata.labels or {}).get("type") == "s3":
-                if self.config.S3_SECRET_PREVIEW == i.metadata.name:
-                    return Minio(
-                        endpoint=base64.b64decode(i.data["S3_HOST"]).decode("utf-8"),
-                        access_key=base64.b64decode(i.data["S3_ACCESS_KEY"]).decode("utf-8"),
-                        secret_key=base64.b64decode(i.data["S3_SECRET_KEY"]).decode("utf-8"),
-                        region=base64.b64decode(i.data["S3_REGION"]).decode("utf-8"),
-                    )
